/************************************************************************************************************
 * (c) Triple-S GmbH 2012
 ************************************************************************************************************/
/*****************************************************************************
*   $Logfile: /SOURCE/Scheduler.c $
*   $Revision: 1 $
*   $Author: Scheduler.x2x$
*   $Date: 2012.11.28 13:13:13 $
*
*    Project:                   TEST
*
*    Content:                   Implementierung der Funktionen fr das Modul Scheduler
*
*
*    Author(s):                 Scheduler.x2x
*
*****************************************************************************/

/*****************************************************************************
*
* CHANGE LOG:
*
* Date     Name Change description
* -----------------------------------------------------------------
*
*****************************************************************************/

/*****************************************************************************
*    Includes
*****************************************************************************/
#include "Project_Defines.h"
#include "Globals.h"

#ifdef SCHEDULER

#define SCHEDULER_C
#include "Scheduler.h"
#undef SCHEDULER_C

/*****************************************************************************
*    Local Defines
*****************************************************************************/
#define BASIC_TIMER_x2        (0x01)
#define BASIC_TIMER_x4        (0x02)
#define BASIC_TIMER_x8        (0x04)
#define BASIC_TIMER_x16       (0x08)
#define BASIC_TIMER_x32       (0x10)
#define BASIC_TIMER_x64       (0x20)
#define BASIC_TIMER_x128      (0x40)
#define BASIC_TIMER_x256      (0x80)

/*---- Internal Macros -----------------------------------------------------*/
/* MISRA[4:3453]: Used for better readability */
#define TIME_SLICE_CALL( func ) ((func)())

/*****************************************************************************
*    Type definitions
*****************************************************************************/

/*****************************************************************************
*    Global variables
*****************************************************************************/

/*****************************************************************************
*    Local variables
*****************************************************************************/

/*****************************************************************************
*    Local Prototypes
*****************************************************************************/

static void TimerTick_BASEx1( void );
static void TimerTick_BASEx5( void );
static void TimerTick_BASEx25( void );

/*****************************************************************************
*   Functions
*****************************************************************************/

/*****************************************************************************
*
* Name          Scheduler_TimeSlice_BASE
*
* Author        Josef Scheduler.x2x
*
* Remark        implements entry function cyclically called by OS
*
*****************************************************************************/

void Scheduler_TimeSlice_BASE(void)
{
	static uint08 u08_GenericTimerCounter_BASEx5=1;
	static uint08 u08_GenericTimerCounter_BASEx25=1;

	BEGIN_TIMESLICE()


	/* 5ms : 5x1x1 ms */
	Can_MainFunction_Write();
	Can_MainFunction_Read();
	FrIf_MainFunction_0();
	LinIf_MainFunction();
	CanNm_MainFunction();
	FrNm_MainFunction_0();
	CanSM_MainFunction();
	FrSM_MainFunction_0();
	FrTp_MainFunction();
	CanTp_MainFunction();
	Com_MainFunctionRx();
	Com_MainFunctionRouteSignals();
	Com_MainFunctionTx();

	TimerTick_BASEx1();


	/* check for TimerTick_BASEx5 */
	if (5 == u08_GenericTimerCounter_BASEx5)
	{
		TimerTick_BASEx5();
		u08_GenericTimerCounter_BASEx5 = 1;
	}
	else
	{
		u08_GenericTimerCounter_BASEx5++;
	}


	/* check for TimerTick_BASEx25 */
	if (25 == u08_GenericTimerCounter_BASEx25)
	{
		/* 125ms : 5x25x1 ms */
		ComCtrl_CarCfg_Run();

		TimerTick_BASEx25();
		u08_GenericTimerCounter_BASEx25 = 1;
	}
	else
	{
		u08_GenericTimerCounter_BASEx25++;
	}

	END_TIMESLICE()

}//void Scheduler_TimeSlice_BASE(void)


/*****************************************************************************
*
* Name          TimerTick_BASEx1
*
* Author        Josef Scheduler.x2x
*
* Remark        implements a simulated timer tick
*
*****************************************************************************/

static void TimerTick_BASEx1( void )
{
	static uint08 u08_TimerCounter_BASEx1 = 0;

	if ((BASIC_TIMER_x2 & u08_TimerCounter_BASEx1) != 0)
	{
		/* 10ms : 5x1x2 ms */
		LinSM_MainFunction();
		Xcp_MainFunction();
		Can_MainFunction_Mode();
		Fee_MainFunction();
		NvM_MainFunction();
		CddTpms_MainFunction();
		EcuM_MainFunction();
		Run_ExteriorLight_Sensor();
		Run_TirePMon_Cyclic();
	}
	else if ((BASIC_TIMER_x4 & u08_TimerCounter_BASEx1) != 0)
	{
		/* 20ms : 5x1x4 ms */
		Run_InteriorLight_Actuator();
		Run_InteriorLight_Sensor();
		Run_SoundSignaling_Sensor();
		Run_SoundSignaling_Actuator();
		Run_HornMgr_Step();
	}
	else if ((BASIC_TIMER_x8 & u08_TimerCounter_BASEx1) != 0)
	{
		/* 40ms : 5x1x8 ms */
		Run_SoundSignaling_Step();
		Run_InteriorLight_Step();
	}
	else
	{
		/*to avoid misra warning*/
	}

    /* check for counter reset */
    if (0xFF == u08_TimerCounter_BASEx1)
    {
        u08_TimerCounter_BASEx1 = 0;
    }
    else
    {
        u08_TimerCounter_BASEx1++;
    }
}//TimerTick_BASEx1

/*****************************************************************************
*
* Name          TimerTick_BASEx5
*
* Author        Josef Scheduler.x2x
*
* Remark        implements a simulated timer tick
*
*****************************************************************************/

static void TimerTick_BASEx5( void )
{
	static uint08 u08_TimerCounter_BASEx5 = 0;

	if ((BASIC_TIMER_x2 & u08_TimerCounter_BASEx5) != 0)
	{
		/* 50ms : 5x5x2 ms */
		Run_ComCtrl_Handler();
		Run_AntiTheftProtection_Sensor();
		Run_AlrmMgr_Step();
	}
	else if ((BASIC_TIMER_x4 & u08_TimerCounter_BASEx5) != 0)
	{
		/* 100ms : 5x5x4 ms */
		ComCtrl_TimeoutCheck_Run();
		Run_LowVoltage_Sensor();
	}
	else if ((BASIC_TIMER_x8 & u08_TimerCounter_BASEx5) != 0)
	{
		/* 200ms : 5x5x8 ms */
		Run_MirrTintgMgr_Step();
		Run_DstBkpInBody_Step();
		Run_HdlmpMon_Step();
	}
	else
	{
		/*to avoid misra warning*/
	}

    /* check for counter reset */
    if (0xFF == u08_TimerCounter_BASEx5)
    {
        u08_TimerCounter_BASEx5 = 0;
    }
    else
    {
        u08_TimerCounter_BASEx5++;
    }
}//TimerTick_BASEx5

/*****************************************************************************
*
* Name          TimerTick_BASEx25
*
* Author        Josef Scheduler.x2x
*
* Remark        implements a simulated timer tick
*
*****************************************************************************/

static void TimerTick_BASEx25( void )
{
	static uint08 u08_TimerCounter_BASEx25 = 0;

	if ((BASIC_TIMER_x2 & u08_TimerCounter_BASEx25) != 0)
	{
		/* 250ms : 5x25x2 ms */
		ComCtrl_VIN_Run();
	}
	else if ((BASIC_TIMER_x4 & u08_TimerCounter_BASEx25) != 0)
	{
		/* 500ms : 5x25x4 ms */
		Run_BattMgr_Step();
		Run_PowerCheck_Step();
	}
	else if ((BASIC_TIMER_x8 & u08_TimerCounter_BASEx25) != 0)
	{
		/* 1000ms : 5x25x8 ms */
		Run_CarCfgDet_Step();
		Run_SteerColCtrl_Step();
	}
	else
	{
		/*to avoid misra warning*/
	}

    /* check for counter reset */
    if (0xFF == u08_TimerCounter_BASEx25)
    {
        u08_TimerCounter_BASEx25 = 0;
    }
    else
    {
        u08_TimerCounter_BASEx25++;
    }
}//TimerTick_BASEx25

#endif
